"""
    formelsammlung.venv_utils
    ~~~~~~~~~~~~~~~~~~~~~~~~~

    Utility function for working with virtual environments.

    :copyright: 2020 (c) Christian Riedel
    :license: GPLv3, see LICENSE file for more details
"""  # noqa: D205, D208, D400
import os
import shutil
import sys

from pathlib import Path
from typing import Optional, Tuple, Union


def get_venv_path() -> Optional[str]:
    """Get path to the venv from where the python executable runs.

    :return: Return venv path or None if python is not called from a venv.
    """
    if hasattr(sys, "real_prefix"):
        return sys.real_prefix  # type: ignore[no-any-return,attr-defined] # pylint: disable=E1101
    if sys.base_prefix != sys.prefix:
        return sys.prefix
    return None


def get_venv_site_packages_dir(venv_path: Union[str, Path]) -> Path:
    """Return path to site-packages dir of given venv.

    :param venv_path: Path to venv
    :return: Path to site-packages dir
    """
    return list(Path(venv_path).glob("**/site-packages"))[0]


def where_installed(program: str) -> Tuple[int, Optional[str], Optional[str]]:
    """Find installation locations for given program.

    Return exit code and locations based on found installation places.
    Search in current venv and globally.

    Exit codes:

    - 0 = nowhere
    - 1 = venv
    - 2 = global
    - 3 = both

    :param program: Program to search
    :return: Exit code, venv executable path, glob executable path
    """
    exit_code = 0

    exe = shutil.which(program)
    if not exe:
        return exit_code, None, None

    venv_path = get_venv_path()
    bin_dir = "\\Scripts" if sys.platform == "win32" else "/bin"
    path_wo_venv = os.environ["PATH"].replace(f"{venv_path}{bin_dir}", "")
    glob_exe = shutil.which(program, path=path_wo_venv)

    if glob_exe is None:
        exit_code += 1
    elif exe == glob_exe:
        exit_code += 2
        exe = None
    else:
        exit_code += 3
    return exit_code, exe, glob_exe



"""
    formelsammlung.tox_env_exe_runner
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    Call tools from tox environments.

    :copyright: 2020 (c) Christian Riedel
    :license: GPLv3, see LICENSE file for more details
"""  # noqa: D205, D208, D400
import subprocess  # nosec
import sys

from pathlib import Path
from typing import List, Optional


def env_exe_runner(
    runner: str, envs: List[str], tool: str, tool_args: Optional[List[str]] = None
) -> int:
    """Call given ``tool`` from given ``tox`` or ``nox`` env considering OS.

    :param runner: 'nox' or 'tox'
    :param envs: List of environments to search the ``tool`` in.
    :param tool: Name of the executable to run.
    :param tool_args: Arguments to give to the ``tool``.
    :return: Exit code 127 if no executable is found or the exit code of the called cmd.
    """
    if runner not in ("tox", "nox"):
        print("No valid runner was given. Choose either 'tox' or 'nox'.")
        return 127

    is_win = sys.platform == "win32"

    exe = Path(f"Scripts/{tool}.exe") if is_win else Path(f"bin/{tool}")
    cmd = None

    if not tool_args:
        tool_args = []

    for env in envs:
        path = Path(f".{runner}") / env / exe
        if path.is_file():
            cmd = (str(path), *tool_args)

    if cmd is None:
        print(
            f"No '{tool}' executable found. Make sure one of the "
            f"following `{runner}` envs is accessible: {envs}"
        )
        return 127

    return subprocess.call(cmd)  # nosec


def cli_caller() -> int:
    """Warp ``env_exe_runner`` to be callable by cli.

    Script to call executables in `tox`/`nox` envs considering OS.

    The script takes two mandatory arguments:

    1. The runner managing the env: `tox` or `nox`.
    2. A string with comma separated `tox`/`nox` envs to check for the executable.
       The envs are checked in given order.
    3. The executable to call like e.g. `pylint`.

    All other arguments after are passed to the tool on call.

    :return: Exit code from ``env_exe_runner``
    """
    return env_exe_runner(
        sys.argv[1], sys.argv[2].split(","), sys.argv[3], sys.argv[4:]
    )  # pragma: no cover


if __name__ == "__main__":
    sys.exit(cli_caller())
